
#include "stdafx.h"
#include <assert.h>

#include "War170API.h"
//#include "WarCmdArg.h"
#include "Commands.h"

using namespace std;

commandlist_t Commands;
extern WARAPI_SESSION_HANDLE hSession;
extern int VerboseFlag;
extern int DebugFlag;


const char *ParamNames[] = 
{
    {"void"}, // unused
    {"arg"}, // Argument without keyword=
    {"group"},
    {"path"},
    {"enable"},
    {"maxconn"},
    {"password"},
    {"when"},
    {"invalid"}
};

const char *CommandName[] = 
{
    {"cmd_adduser"},
    {"cmd_deluser"},
    {"cmd_shutdown"},
    { NULL }
};

const param_table_t ParamTable[] = 
{
    {"group",   param_group},
    {"path",    param_path},
    {"enable",  param_enable},
    {"maxconn", param_maxconn},
    {"password",param_password},
    {"when",    param_when},
    {NULL, param_invalid}
};

const command_t CommandTable[] = 
{
    {"adduser", cmd_adduser, 
        {param_arg, param_group}, 
        {param_path, param_enable, param_maxconn, param_password}
    },

    {"deluser", cmd_deluser, {param_arg}, {param_void}},
    {"shutdown", cmd_shutdown, {param_void}, {param_when}},
    {NULL, cmd_invalid, {param_void}, {param_void}}
};

// Check that arguments are sane
void Paramater::Validate()
{
    switch(m_ID)
    {
    case param_arg:
        break;
    case param_group:
        break;
    case param_path:
        CheckPath();
        break;
    case param_enable:
        CheckEnable();
        break;
    case param_maxconn:
        break;
    case param_password:
        break;
    case param_when:
        CheckWhen();
        break;
    case param_invalid:
    case param_void:
        assert(false);
    }
}

void Paramater::Execute(WARAPI_USER_HANDLE& V)
{
    switch(m_ID)
    {
    case param_password:
        SetPassword(V);
        break;
    case param_path:
        SetPath(V);
        break;
    case param_enable:
        SetEnable(V);
        break;
    }
}

void Paramater::CheckEnable()
{
    if (m_Args.size() != 1)
    {
        cerr << "Bad number of arguments in enable option" << endl;
        exit(0);
    }

    if (stricmp(m_Args[0].c_str(), "yes") 
        && stricmp(m_Args[0].c_str(), "no")
        && stricmp(m_Args[0].c_str(), "default"))
    {
        cerr << "Unknown arguments in enable option :" << m_Args[0].c_str() << endl;
        exit(0);
    }
}

void Paramater::CheckWhen()
{
    if (m_Args.size() != 1)
    {
        cerr << "Bad number of arguments in when option" << endl;
        exit(0);
    }

    if (stricmp(m_Args[0].c_str(), "now") && stricmp(m_Args[0].c_str(), "wait"))
    {
        cerr << "Unknown arguments in when option :" << m_Args[0].c_str() << endl;
        exit(0);
    }
}

void Paramater::CheckPath()
{
    if (m_Args.size() != 3)
    {
        cerr << "Bad number of arguments in path option" << endl;
        exit(0);
    }

    const char *p = m_Args[2].c_str();
    {
        switch(*p)
        {
        case 'r': case 'w': case 'e': case 'l': case 'd': case 'R': case 'C': 
        case 'h': case 'D': case 'F': case 'E': case 'm':
            break;
        default:
            cerr << "Bad flag in path option. " 
                << *p 
                << " is not recognized as a valid flag." 
                << endl;
            exit(-1);
        }
        ++p;
    }
}

void Paramater::SetEnable(WARAPI_USER_HANDLE& V)
{
    int Err = 0;
    WARAPI_LOGIN_PERM Perm = WARAPI_LOGIN_PERM_DEFAULT;

    if (!stricmp(m_Args[0].c_str(), "yes"))
        Perm = WARAPI_LOGIN_PERM_ALLOW;
    else if (!stricmp(m_Args[0].c_str(), "no"))
        Perm = WARAPI_LOGIN_PERM_DENY;

    if (Warapi_SetLoginPermission(hSession,
        V,
        WARAPI_LOGIN_TYPE_FTPUSER,
        Perm,
        &Err) != WARAPI_CALL_OK)
    {
        cerr << "Failed to set Login permission to " << m_Args[0].c_str() << ". Error " << Err << endl;
        exit(-1);
    }
}

void Paramater::SetPath(WARAPI_USER_HANDLE& V)
{
    WARAPI_FLAGS Flags = 0;
    int Err = 0;

    const char *p = m_Args[2].c_str();
    while(*p)
    {
        switch(*p)
        {
        case 'r': Flags |= WARAPI_PERM_READ; break;
        case 'w': Flags |= WARAPI_PERM_WRITE; break;
        case 'e': Flags |= WARAPI_PERM_EXECUTE; break;
        case 'l': Flags |= WARAPI_PERM_LIST; break;
        case 'd': Flags |= WARAPI_PERM_DELETE; break;
        case 'm': Flags |= WARAPI_PERM_REMOVE; break;
        case 'C': Flags |= WARAPI_PERM_CREATE; break;
        case 'h': Flags |= WARAPI_PERM_HIDE; break;
        case 'R': Flags |= WARAPI_PERM_RECURSIVE; break;
        case 'D': Flags |= WARAPI_PERM_DENY; break;
        case 'F': Flags |= WARAPI_PERM_FREE; break;
        case 'E': Flags |= WARAPI_PERM_READ
                        |  WARAPI_PERM_LIST
                        |  WARAPI_PERM_RECURSIVE; break;
        }
        ++p;
    }
    
    if (Warapi_SetPath(hSession,
        V,
        m_Args[0].c_str(),
        m_Args[1].c_str(),
        Flags,
        &Err) != WARAPI_CALL_OK)
    {
        cerr << "Failed to set the path " << m_Args[0].c_str() << ". Error " << Err << endl;
        exit(-1);
    }

}

void Paramater::SetPassword(WARAPI_USER_HANDLE& V)
{
    int Err = 0;

    assert(m_Args.size() == 1);
    WARAPI_CCSTR_T Passwd = m_Args[0].c_str();
    if (!Passwd)
        Passwd = "";

    if (Warapi_SetPassword(hSession,V,Passwd,&Err) != WARAPI_CALL_OK)
    {
        cerr << "Failed to set new password. Error " << Err << endl;
        exit(-1);
    }
}

void Command::Execute()
{
    switch(m_ID)
    {
    case cmd_adduser:
        AddUser();
        break;
    case cmd_deluser:
        DelUser();
        break;
    case cmd_shutdown:
        Shutdown();
        break;
    default:
        cerr << "The command " << CommandName[m_ID] << " is not yet implemented." << endl;
        exit(-1);
    }
}


void Command::AddUser()
{
    WARAPI_USER_HANDLE hUser;
    std::string Parent, Name;
    int Err = 0;

    // Get infor to add user
    paramlist_t::iterator P = m_Params.begin();
    for(; P != m_Params.end()
        ; P++)
    {
        switch(P->m_ID)
        {
        case param_group:
            Parent = P->m_Args[0];
            break;
        case param_arg:
            Name = P->m_Args[0];
            break;
        }
    }

    assert(!Parent.empty());

    if (Warapi_CreateUser(hSession,
        Parent.c_str(),
        Name.c_str(),
        &hUser,
        &Err) != WARAPI_CALL_OK)
    {
        cerr << "Cound not add a new user. Error: " << Err << endl;
        exit(-1);
    }

    if (VerboseFlag)
        cout << "Added user " << Name.c_str() << endl;

    // Process optional parameters
    // The param_group and param_arg parameters ingnores the Execute() fuction.
    for(P = m_Params.begin()
        ; P != m_Params.end()
        ; P++)
    {
        P->Execute(hUser);
    }

    Warapi_ReleaseUserHandle(hUser);
}

void Command::DelUser()
{
    int Err = 0;
    std::string Name;

    for(paramlist_t::iterator P = m_Params.begin()
        ; P != m_Params.end()
        ; P++)
    {
        switch(P->m_ID)
        {
        case param_arg:
            Name = P->m_Args[0];
            break;
        }
    }

    if (Warapi_DeleteUser(hSession, Name.c_str(), &Err) != WARAPI_CALL_OK)
    {
        cerr << "Cound not delete node " << Name.c_str() << ". Error: " << Err << endl;
        exit(-1);
    }

    cout << Name.c_str() << " deleted!" << endl;
}

void Command::Shutdown()
{
    WARAPI_BOOL_T Relay = 0;
    int Err = 0;

    for(paramlist_t::iterator P = m_Params.begin()
        ; P != m_Params.end()
        ; P++)
    {
        switch(P->m_ID)
        {
        case param_when:
            if (!stricmp(P->m_Args[0].c_str(), "wait"))
                Relay = 1;
            break;
        }
    }

    if (Warapi_Shutdown(hSession, Relay, &Err) != WARAPI_CALL_OK)
    {
        cerr << "Failed to initiate Shutdown. Error " << Err << endl;
        exit(-1);
    }

    cout << "Shutdown was initiated" << endl;
}