

enum CommandsE
{
    cmd_adduser,
    cmd_deluser,
    cmd_shutdown,
    cmd_invalid
};

enum ParamE
{
    param_void, // unused
    param_arg, // Argument without keyword=
    param_group,
    param_path,
    param_enable,
    param_maxconn,
    param_password,
    param_when,
    param_invalid
};

typedef struct command_def
{
    const char *name;
    CommandsE cmd;
    ParamE req_params[param_invalid +1];
    ParamE opt_params[param_invalid +1];
} command_t;

typedef struct param_table_def
{
    const char *name;
    ParamE param;
} param_table_t;

extern const char *ParamNames[];
extern const param_table_t ParamTable[];
extern const command_t CommandTable[];

class Paramater
{
public:
    typedef std::vector<std::string> arglist_t;

    Paramater(const ParamE& ID)
        : m_ID(ID) {}

    Paramater(const Paramater& V)
    {
        operator = (V);
    }

    Paramater& operator = (const Paramater& V)
    {
        m_ID = V.m_ID;
        m_Args = V.m_Args;
        return *this;
    }

    void Validate();
    void Execute(WARAPI_USER_HANDLE& V);
    
    ParamE m_ID;
    arglist_t m_Args;
private:
    void CheckEnable();
    void CheckWhen();
    void CheckPath();
    void SetEnable(WARAPI_USER_HANDLE& V);
    void SetPath(WARAPI_USER_HANDLE& V);
    void SetPassword(WARAPI_USER_HANDLE& V);
};

class Command
{
public:
    typedef std::list<Paramater> paramlist_t;
    
    Command(const CommandsE ID)
        : m_ID(ID) {}

    Command(const Command& V)
    {
        operator = (V);
    }

    Command& operator = (const Command& V)
    {
        m_ID = V.m_ID;
        m_Params = V.m_Params;
        return *this;
    }

    void Execute();

    CommandsE m_ID;
    paramlist_t m_Params;

protected:
    void AddUser();
    void DelUser();
    void Shutdown();
};

typedef std::list<Command> commandlist_t;

extern commandlist_t Commands;


