// WarFtpdUtil.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"

#include "War170API.h"
#include "WarCmdArg.h"
#include "Commands.h"

using namespace std;

#define PROGRAM "warftpdutil"
#define VERSION "1.0"
#define COPYRIGHT "Copyright 2000 by Jarle (jgaa) Aase"

string Host;
string UserName;
string Password;
unsigned short Port = 0;
WARAPI_SESSION_HANDLE hSession;

int VerboseFlag;
int DebugFlag;
int BatchFlag;

void Usage(const char *BadArg = NULL);


void ParseArgs(int argc, char **argv)
{
    // Use the WarCmdArg class to parse args
    // First, define return values for the args that
    // don't have one-character substitutes.
    enum ArgsE
    {
        SHOW_VERSION = 256
    };

    // Define the long option names
    static const WarCmdArg::option opts[] = 
    {
        {"version", 0, NULL, SHOW_VERSION},
        {"host", WarCmdArg::required_argument, NULL, 'H'},
        {"port", WarCmdArg::required_argument, NULL, 'P'},
        {"login", WarCmdArg::required_argument, NULL, 'l'},
        {"password", WarCmdArg::required_argument, NULL, 'p'},
        {"verbose", 0, NULL, 'v'},
        {"debug", WarCmdArg::optional_argument, NULL, 'd'},
        {"help", 0, NULL, 'h'},
        {"batch", 0, NULL, 'b'},
        {NULL, 0, NULL, 0}
    };

    // Create the options parser
    WarCmdArg Args(argc, argv, "bvdhH:p:l:p:", opts);

    // Parse arguments
    int Opt = 0;
    while((Opt = Args.GetArg()) > 0)
    {
        switch(Opt)
        {
        case SHOW_VERSION:
            cout << PROGRAM << " " << VERSION << " " << __DATE__ << endl;
            exit(-1);
        case 'H':
            Host = Args.GetArgument();
            break;
        case 'P':
            Port = (unsigned short)atoi(Args.GetArgument());
            break;
        case 'h':
            Usage();
        case 'v':
            VerboseFlag++;
            break;
        case 'd':
            if (Args.GetArgument())
            {
                DebugFlag = atoi(Args.GetArgument());
                if (DebugFlag < 1)
                    Usage(Args.GetCurrArg());
            }
            else
                DebugFlag++;
            break;
        case 'l':
            UserName = Args.GetCurrArg();
            break;
        case 'p':
            Password = Args.GetCurrArg();
            break;
        case 'b':
            BatchFlag = true;
            break;
        case '?':
            Usage();
        }
    }

    int MyArgc = Args.GetOptind();
    int Index = 0;
    const command_t *pCmd = NULL;

    if (MyArgc >= argc)
    {
        cerr << "Missing command identifier.";
        exit(-1);
    }

    // Get command verb
    for(Index = 0; CommandTable[Index].cmd != cmd_invalid; Index++)
    {
        if (!strcmp(argv[MyArgc], CommandTable[Index].name))
        {
            pCmd = &CommandTable[Index];
            break;
        }
    }

    if (!pCmd)
    {
        cerr << argv[MyArgc] << " is not a recognized as a command" << endl;
        exit(-1);
    }
    
    Commands.push_back(Command(pCmd->cmd));

    // Get command parameters
    while(++MyArgc < argc)
    {
        int ArgIndex = 0;
        for(Index = 0; ParamTable[Index].param != param_invalid; Index++)
        {
            const char *p = ParamTable[Index].name;
            const char *pp = argv[MyArgc];

            while(*p && (*p == *pp))
                ++p, ++pp;
    
            if (!*p && (*pp == '='))
            {
                Paramater Param(ParamTable[Index].param);

                // Get comma seperated arguments to parameter
                while(*pp)
                {
                    string arg;
                    while(*++pp)
                    {
                        if (*pp == ',')
                            break;

                        arg += *pp;
                    }

                    if (!arg.empty())
                        Param.m_Args.insert(Param.m_Args.end(), arg);
                }

                Param.Validate();
                Commands.front().m_Params.push_back(Param);
                goto next_arg;
            }
            if (!strchr(argv[MyArgc], '='))
            {
                if (pCmd->req_params[ArgIndex] == param_arg)
                {
                    Paramater Param(param_arg);
                    Param.m_Args.insert(Param.m_Args.end(), argv[MyArgc]);
                    Commands.front().m_Params.push_back(Param);
                    ++ArgIndex;
                    goto next_arg;
                }
            }
        }

        cerr << argv[MyArgc] << " is not recognized as a valid argument in this context." << endl;
        exit(-1);

next_arg:
        ;
    }

    // Check if all requiered arguments are supplied
    
    for(Index = 0; pCmd->req_params[Index]!= param_void; Index++)
    {
        Command::paramlist_t::const_iterator P = Commands.front().m_Params.begin();
        for(
            ; P != Commands.front().m_Params.end()
            ; P++)
        {
            const Paramater& _P = *P;
            if (_P.m_ID == pCmd->req_params[Index])
                break;
        }

        if (P == Commands.front().m_Params.end())
        {
            cerr << "Missing requiered parameter " << ParamNames[Index] << endl;
            exit(-1);
        }
    }

    // Seems ok.
}

int main(int argc, char* argv[])
{
    int Err = 0;

	if (argc < 2)
        Usage();

    ParseArgs(argc, argv);

    if (UserName.empty())
    {
        if (!BatchFlag)
        {
            char Buf[33];
            cout << "Username [Sysadmin]: ";
            cin.getline(Buf, sizeof(Buf) -1);
            UserName = Buf;
            cout << endl;
            
        }
        if (UserName.empty())
            UserName = "Sysadmin";
    }
    
    if (Password.empty())
    {
        if (!BatchFlag)
        {
            char Buf[33];
            cout << "Password: ";
            cin.getline(Buf, sizeof(Buf) -1);
            Password = Buf;
        }

        if (Password.empty())
        {
            cerr << "No password provided. Aborting." << endl;
            return -1;
        }
    }

    if (Warapi_InitializeApi(WAR_API_INTERFACE_VERISON, &Err) != WARAPI_CALL_OK)
    {
        cerr << PROGRAM << ": FATAL ERROR - Unable to initialize the interface DLL." << endl;
        return -1;
    }

    if (Warapi_OpenSession(Host.empty() ? "localhost" : Host.c_str(),
        Port,
        UserName.c_str(),
        Password.c_str(),
        &hSession,
        &Err) != WARAPI_CALL_OK)
    {
        cerr << "Error: Could not connect to server (error " 
            << Err
            << "). Aborting." << endl;
        if (Err == WARAPI_ERR_AUTH)
            cerr << "Bad username or password" << endl;
        return -1;
    }

    for(commandlist_t::iterator P = Commands.begin()
        ; P != Commands.end()
        ; P++)
    {
        P->Execute();
    }

    Warapi_CloseSession(hSession);

    return 0; // All ok
}

void Usage(const char *BadArg)
{
    cout << PROGRAM << " " << VERSION << " " << COPYRIGHT << endl
        << "Usage: " << PROGRAM << " [options] command {args} ... " << endl
        << "Options: " << endl;
    cout << "   -l Name     --login=Name      Login name" << endl;
    cout << "   -p Passwd   --password=Passwd Login Password" << endl;
    cout << "   -H Host     --host=Host       Remote host to use" << endl;
    cout << "   -P Port     --port=Port       Port to use" << endl;
    cout << "   -d          --debug=Level     Debug level" << endl;
    cout << "   -v          --verbose         Verbose output" << endl;
    cout << "   -L File     --log-file=File   Optional log file " << endl;
    cout << "   -b          --batch           Do not prompt for missing arguments" << endl;
    cout << "   -h          --help            Show this help" << endl;
    cout << "               --version         Show program version" << endl;
    cout << "Commands:" << endl;
    cout << "   adduser name group=group      Add a user to an existig group" << endl;
    cout << "       [password=passwd]         Set password" << endl;
    cout << "       [path=vpath,ppath,perms]  Define a path" << endl;
    cout << "           vpath is mount point, like / or /home" << endl;
    cout << "           ppath is physical path, like c:\\ftp" << endl;
    cout << "           flags are: E default, r read, w write, e execute," << endl;
    cout << "                      l list, d delete, m remove dir, C creadte dir," << endl;
    cout << "                      h hide, R recursive, D demy, F free" << endl;
    cout << "       [enable=yes|no|default]   Enable or disable login" << endl;
    cout << "   deluser name                  Delete a user" << endl;
    cout << "   shutdown [when=now|wait]      Shut down. If when=wait, current" << endl;
    cout << "                                 transfers will be allowed to complete." << endl;
    cout << "Example:" << endl;
    cout << "   " << PROGRAM << " -bvp test adduser johndoe group=/system/user path=/home,h:=\\user\\johndoe,Ewc enable=yes password=secret" << endl;
    cout << "   " << PROGRAM << " deluser /system/user/johndoe" << endl;
    cout << endl;
    cout << PROGRAM << " --help | more        to view all the options" << endl;

    exit(-1);
}

